from org.ohdsi.circe.cohortdefinition.printfriendly import MarkdownRender

from jpype.types import JObject, JArray


def cohort_print(expression: JObject):
    """
    Create a print friendly version of a cohort expression.

    Parameters
    ----------
    expression : JObject
        A character vector or result of ``cohort_expression_from_json``
        containing the cohort expression.

    Returns
    -------
    str
        A character vector containing the print friendly version of the
        cohort expression.
    """
    return MarkdownRender().renderCohort(expression)


def concept_set_print(expression: JObject):
    """
    Create a print friendly version of a concept set expression.

    Parameters
    ----------
    expression : JObject
        A character vector or result of ``concept_set_expression_from_json``
        containing the concept set expression.

    Returns
    -------
    str
        A character vector containing the print friendly version of the
        concept set expression.
    """
    return MarkdownRender().renderConceptSet(expression)


def concept_set_list_print(expression: JObject | list[JObject] | JArray):
    """
    Render conceptSet array for print-friendly

    Generates a print-friendly (human-readable) representation of an array of
    concept sets. This can for example be used in a study protocol.

    Parameters
    ----------
    expression : JObject | list[JObject] | JArray
        A character vector or result of ``concept_set_expression_from_json``
        containing the concept set expression.


    Returns
    -------
    str
        A character vector containing the print friendly version of the
        concept set expression.
    """
    return MarkdownRender().renderConceptSetList(expression)



import json

from org.ohdsi.circe.vocabulary import ConceptSetExpression as \
    JavaConceptSetExpression
from jpype.types import JClass, JObject


def ConceptSetExpression() -> JClass:
    """
    Return the ConceptSetExpression class.

    Returns
    -------
    JClass
        A org.ohdsi.circe.vocabulary.ConceptSetExpression class.
    """
    return JavaConceptSetExpression


def concept_set_expression_from_json(concept_set: str | dict) -> JObject:
    """
    Render read JSON into a ConceptSetExpression instance.

    Reads a String (json) and deserializes it into a
    org.ohdsi.circe.vocabulary.ConceptSetExpression

    Parameters
    ----------
    concept_set : str | dict
        A JSON string representing a concept set expression. Or a dictionary
        representing a concept set expression.

    Returns
    -------
    JObject
        A java instance of
        ``org.ohdsi.circe.vocabulary.ConceptSetExpression``.
    """
    if isinstance(concept_set, dict):
        concept_set = json.dumps(concept_set)

    return JavaConceptSetExpression.fromJson(concept_set)



from jpype.types import JInt
from jpype.types import JClass, JObject
from org.ohdsi.circe.cohortdefinition import CohortExpressionQueryBuilder as \
    JavaCohortExpressionQueryBuilder
from org.ohdsi.circe.cohortdefinition.CohortExpressionQueryBuilder import \
    BuildExpressionQueryOptions as JavaBuildExpressionQueryOptions


def CohortExpressionQueryBuilder() -> JClass:
    """
    Return the CohortExpressionQueryBuilder class.

    Returns
    -------
    _JClass
        A org.ohdsi.circe.cohortdefinition.CohortExpressionQueryBuilder class.
    """
    return JavaCohortExpressionQueryBuilder


def create_generate_options(
        chohort_id_field_name: str = None, cohort_id: int = None,
        cdm_schema: str = None, target_table: str = None,
        result_schema: str = None, vocabulary_schema: str = None,
        generate_stats: bool = None) -> JObject:
    """
    Create Generation Options

    Creates the generation options object for use in ``build_cohort_query``.

    Parameters
    ----------
    chohort_id_field_name : str, optional
        The field that contains the cohortId in the cohort table, by default
        None
    cohort_id : int, optional
        The generated cohort ID, by default None
    cdm_schema : str, optional
        The value of the CDM schema, by default None
    target_table : str, optional
        The cohort table name, by default None
    result_schema : str, optional
        The schema the cohort table belongs to, by default None
    vocabulary_schema : str, optional
        The schema of the vocabulary tables (defaults to cdmSchema),
        by default None
    generate_stats : bool, optional
        A boolean representing if the query should include inclusion rule
        statistics calculation, by default None

    TODO
    ----
    Somehow the class expect a $ in the name of the class, weirdness..
    TypeError: No matching overloads found for *static* org.ohdsi.circe.cohortdefinition.CohortExpressionQueryBuilder.buildExpressionQuery(org.ohdsi.circe.cohortdefinition.CohortExpression,org.ohdsi.circe.cohortdefinition.CohortExpressionQueryBuilder.BuildExpressionQueryOptions), options are:
        public java.lang.String org.ohdsi.circe.cohortdefinition.CohortExpressionQueryBuilder.buildExpressionQuery(org.ohdsi.circe.cohortdefinition.CohortExpression,org.ohdsi.circe.cohortdefinition.CohortExpressionQueryBuilder$BuildExpressionQueryOptions)
        public java.lang.String org.ohdsi.circe.cohortdefinition.CohortExpressionQueryBuilder.buildExpressionQuery(java.lang.String,org.ohdsi.circe.cohortdefinition.CohortExpressionQueryBuilder$BuildExpressionQueryOptions)

    Returns
    -------
    _JObject
        _description_
    """
    options = JavaBuildExpressionQueryOptions()

    if chohort_id_field_name:
        options.cohortIdFieldName = chohort_id_field_name
    if cohort_id:
        options.cohortId = JInt(cohort_id)
    if cdm_schema:
        options.cdmSchema = cdm_schema
    if target_table:
        options.targetTable = target_table
    if result_schema:
        options.resultSchema = result_schema
    if vocabulary_schema:
        options.vocabularySchema = vocabulary_schema
    if generate_stats:
        options.generateStats = generate_stats

    return options


def build_cohort_query(cohort_expression: JObject | str,
                       options: JObject = None) -> str:
    """
    Build Cohort SQL

    Generates the OMOP CDM Sql to generate the cohort expression

    Parameters
    ----------
    cohort_expression : _JObject | str
        A java instance of
        ``org.ohdsi.circe.cohortdefinition.CohortExpression`` or a JSON
        string representing a cohort expression.
    options : _JObject, optional
        The options object from ``create_generate_options``, by default None
    """
    # expression can be a org.ohdsi.circe.cohortdefinition.CohortExpression
    # or a String (it’s an overload method)
    if not options:
        options = create_generate_options()

    sql = JavaCohortExpressionQueryBuilder().buildExpressionQuery(
        cohort_expression, options)

    return sql



import json

from jpype.types import JClass
from org.ohdsi.circe.vocabulary import ConceptSetExpressionQueryBuilder as \
    JavaConceptSetExpressionQueryBuilder

from circe.concept_set_expression import concept_set_expression_from_json


def ConceptSetExpressionQueryBuilder() -> JClass:
    """
    Return the ConceptSetExpressionQueryBuilder class.

    Returns
    -------
    JClass
        A org.ohdsi.circe.vocabulary.ConceptSetExpressionQueryBuilder class.
    """
    return JavaConceptSetExpressionQueryBuilder


def build_concept_set_query(concept_set_expression: str | dict) -> str:
    """
    Build a query from a ConceptSetExpression instance.

    Parameters
    ----------
    concept_set_expression : ConceptSetExpression
        A java instance of
        ``org.ohdsi.circe.vocabulary.ConceptSetExpression``.

    Returns
    -------
    str
        A query string.
    """
    if isinstance(concept_set_expression, dict):
        concept_set_expression = json.dumps(concept_set_expression)

    return JavaConceptSetExpressionQueryBuilder.buildExpressionQuery(
        concept_set_expression_from_json(concept_set_expression)
    )



import jpype
import jpype.imports

from importlib.resources import files


# start the JVM if it hasn't been started yet
jar_folder = files('circe.java')
if not jpype.isJVMStarted():
    jpype.startJVM(classpath=[str(jar_folder.joinpath('*'))])


if __name__ == '__main__':

    from circe.cohort_expression import cohort_expression_from_json
    from circe.print_friendly import cohort_print, concept_set_print, concept_set_list_print
    cohort_json = files('circe.data').joinpath('simpleCohort.json').read_text()
    exp = cohort_expression_from_json(cohort_json)
    print(cohort_print(exp))
    print('*'*80)

    from circe.cohort_sql_builder import build_cohort_query, create_generate_options

    query = build_cohort_query(exp)
    print(query)
    print('*'*80)
    from circe.concept_set_expression import concept_set_expression_from_json

    concept_set_json = files('circe.data').joinpath('conceptSet.json').read_text()
    concept = concept_set_expression_from_json(concept_set_json)
    print(concept_set_print(concept_set_json))
    print('*'*80)

    concept_set_list_json = files('circe.data').joinpath('conceptSetList.json').read_text()
    print(concept_set_list_print(concept_set_list_json))




from setuptools import setup, find_namespace_packages


setup(
    name='python-circe',
    description='Python wrapper for Circe',
    long_description='Python wrapper for Circe',
    long_description_content_type='text/markdown',
    version='0.0.2',
    packages=find_namespace_packages(),
    include_package_data=True,
    package_data={
        'circe': ['*.jar', '*.json'],
    },
    python_requires='>=3.9',
    install_requires=[
        'jpype1==1.4.1',
    ]
)



import json

from jpype.types import JClass, JObject
from org.ohdsi.circe.cohortdefinition import CohortExpression as \
    JavaCohortExpression


def CohortExpression() -> JClass:
    """
    Return the CohortExpression class.

    Returns
    -------
    _JClass
        A org.ohdsi.circe.cohortdefinition.CohortExpression class.
    """
    return JavaCohortExpression


def cohort_expression_from_json(cohort: str | dict) -> JObject:
    """
    Render read JSON into a CohortExpression instance.

    Reads a String (json) and deserializes it into a
    org.ohdsi.circe.cohortdefinition.CohortExpression

    Parameters
    ----------
    json : str | dict
        A JSON string representing a cohort expression. Or a dictionary
        representing a cohort expression.

    Returns
    -------
    _JObject
        A java instance of
        ``org.ohdsi.circe.cohortdefinition.CohortExpression``.
    """
    if isinstance(cohort, dict):
        cohort = json.dumps(cohort)

    return JavaCohortExpression.fromJson(cohort)
